@@ -1,4 +1,3 @@
-# pylint: disable=too-many-lines
 """
 An RSS/Atom syndication feed in an archive.
 """
@@ -10,13 +9,14 @@ import urllib
 import email.utils
 import pathlib
 import logging
+import pdb
 
-from lxml import etree
-from requests_toolbelt.downloadutils import stream
+from lxml import etree  # nosec B410
 
 from . import utils
+from .utils import mimetypes
 from . import formats
-from . import linkpaths
+from . import enclosures
 
 logger = logging.getLogger(__name__)
 
@@ -31,7 +31,8 @@ class ArchiveFeed:
 
     # Initialized when the configuration is loaded prior to update
     url = None
-    link_path_plugins = None
+    enclosure_plugins = None
+    enclosure_fallack_plugins = None
     # Initialized on update from the response to the request for the URL from the feed
     # config in order to use response headers to derrive the best path.
     path = None
@@ -52,18 +53,26 @@ class ArchiveFeed:
         Pre-process and validate the feed config prior to running the actual update.
         """
         self.url = self.config["remote-url"]
-        self.link_path_plugins = self.archive.link_path_plugins + list(
-            linkpaths.load_plugins(self, self.config),
+        self.enclosure_plugins = self.archive.enclosure_plugins[:]
+        self.enclosure_fallack_plugins = self.archive.enclosure_fallack_plugins[:]
+        enclosure_plugins, enclosure_fallack_plugins = enclosures.load_plugins(
+            self,
+            self.config,
         )
+        self.enclosure_plugins.extend(enclosure_plugins)
+        self.enclosure_fallack_plugins.extend(enclosure_fallack_plugins)
 
-    def update(
+    # Sub-commands
+
+    # TODO: Refactor to reduce complexity and improve readability and testibility
+    def update(  # noqa: MC0001
         self,
     ):  # pylint: disable=too-many-locals,too-many-statements,too-many-branches
         """
         Request the URL of one feed in the archive and update contents accordingly.
         """
-        logger.info("Requesting feed: %r", self.url)
-        remote_response = self.archive.requests.get(self.url)
+        logger.debug("Requesting feed: %r", self.url)
+        remote_response = self.archive.client.get(self.url)
         # Maybe update the extension based on the headers
         self.path = self.archive.root_path / self.archive.response_to_path(
             remote_response,
@@ -74,17 +83,17 @@ class ArchiveFeed:
 
         # Assemble the archive version of the feed XML
         download_paths = {}
-        archive_tree = self.load_archive_tree(
-            remote_format,
+        archive_tree = self.get_archive_tree(
             remote_tree,
             download_paths,
+            remote_format=remote_format,
         )
         archive_root = archive_tree.getroot()
         archived_items_parent = remote_format.get_items_parent(archive_root)
 
         remote_item_elems = list(remote_format.iter_items(remote_root))
         archive_item_elems = list(remote_format.iter_items(archive_root))
-        if (len(remote_item_elems) - len(archive_item_elems)) > 4:  # pragma: no cover
+        if (len(remote_item_elems) - len(archive_item_elems)) > 4:
             logger.warning(
                 "Many more items in remote than archive: %s > %s",
                 len(remote_item_elems),
@@ -158,71 +167,145 @@ class ArchiveFeed:
                     str(self.path),
                 )
 
-                item_download_paths = self.download_item_content(
+                item_download_paths = self.download_item_enclosures(
                     remote_format,
                     remote_item_elem,
                     remote_item_id,
                 )
-                if item_download_paths is None:  # pragma: no cover
+                if item_download_paths is None:
                     continue
                 (
-                    item_download_asset_paths,
-                    item_download_content_paths,
+                    item_asset_paths,
+                    item_enclosure_paths,
                 ) = item_download_paths
-                download_paths.update(item_download_asset_paths)
-                download_paths.update(item_download_content_paths)
+                download_paths.update(item_asset_paths)
+                download_paths.update(item_enclosure_paths)
                 updated_items[remote_item_id] = remote_item_elem
 
-                self.link_item_content(
+                self.link_item_enclosures(
+                    remote_format=remote_format,
                     feed_elem=archived_items_parent,
                     item_elem=remote_item_elem,
-                    item_content_paths=item_download_content_paths,
+                    item_enclosure_paths=item_enclosure_paths,
                 )
 
                 archived_items_parent.insert(first_item_idx, remote_item_elem)
-                if updated_items or download_paths:  # pragma: no cover
-                    # Pretty format the feed for readability
-                    etree.indent(archive_tree)
-                    # Update the archived feed file
-                    archive_tree.write(str(self.path))
+                # Pretty format the feed for readability
+                etree.indent(archive_tree)
+                # Update the archived feed file
+                archive_tree.write(str(self.path))
 
         update_download_metadata(remote_response, self.path)
 
-        return list(updated_items.keys()), {
-            # Return values fit for CLI output
-            download_url: str(download_path)
-            for download_url, download_path in download_paths.items()
-        }
+        if updated_items or download_paths:
+            return list(updated_items.keys()), {
+                # Return values fit for CLI output
+                download_url: str(download_path)
+                for download_url, download_path in download_paths.items()
+            }
+        return None
 
-    def download_item_content(self, remote_format, remote_item_elem, remote_item_id):
+    # TODO: Refactor to reduce complexity and improve readability and testibility
+    def relink(self):  # noqa: MC0001
         """
-        Download all the enclosures/content from a feed item.
+        Re-link enclosures to the correct locations for this feed.
         """
-        item_download_asset_urls = formats.all_xpaths_results(
+        # Parse this feed's archive XML
+        self.path = self.find_archive_path()
+        archive_tree = self.load_archive_tree()
+        archive_format = formats.FeedFormat.from_tree(self, archive_tree)
+        attr_prefixes = [
+            f"{{{self.NAMESPACE}}}enclosure-link-",
+            # BBB: old format compatibility
+            f"{{{self.NAMESPACE}}}content-link-",
+        ]
+
+        # Update the links for each item in this feed
+        is_modified = False
+        linked_enclosures = {}
+        for archive_item_elem in archive_format.iter_items(archive_tree.getroot()):
+            item_enclosure_paths = {}
+            for url_result in formats.all_xpaths_results(
+                archive_item_elem,
+                archive_format.DOWNLOAD_ITEM_ENCLOSURE_URLS_XPATHS,
+            ):
+                for attr, attr_value in list(url_result.getparent().attrib.items()):
+                    for attr_prefix in attr_prefixes:
+                        if attr.startswith(attr_prefix):
+                            break
+                    else:
+                        continue
+                    enclosure_path = pathlib.Path(attr_value)
+                    if enclosure_path.is_symlink():
+                        logger.info(
+                            "Deleting existing enclosure link: %r -> %r",
+                            str(enclosure_path),
+                            str(os.readlink(enclosure_path)),
+                        )
+                        enclosure_path.unlink()
+                    del url_result.getparent().attrib[attr]
+                    is_modified = True
+                item_enclosure_paths[url_result] = pathlib.Path(
+                    urllib.parse.unquote(
+                        urllib.parse.urlsplit(url_result).path.lstrip("/"),
+                    ),
+                )
+            item_enclosures = self.link_item_enclosures(
+                remote_format=archive_format,
+                feed_elem=archive_format.get_items_parent(archive_tree.getroot()),
+                item_elem=archive_item_elem,
+                item_enclosure_paths=item_enclosure_paths,
+            )
+            if item_enclosures:
+                is_modified = True
+            if is_modified:  # pragma: no cover
+                # Pretty format the feed for readability
+                etree.indent(archive_tree)
+                # Update the archived feed file
+                archive_tree.write(str(self.path))
+            linked_enclosures.update(
+                (
+                    url_result,
+                    # Make results JSON serializable for CLI stdout
+                    [str(enclosure_path) for enclosure_path in enclosure_paths],
+                )
+                for url_result, enclosure_paths in item_enclosures.items()
+            )
+        if linked_enclosures:
+            return linked_enclosures
+        return None
+
+    # Other methods
+
+    def download_item_enclosures(self, remote_format, remote_item_elem, remote_item_id):
+        """
+        Download all the enclosures from a feed item.
+        """
+        item_asset_urls = formats.all_xpaths_results(
             remote_item_elem,
             remote_format.DOWNLOAD_ITEM_ASSET_URLS_XPATHS,
         )
-        item_download_content_urls = formats.all_xpaths_results(
+        item_enclosure_urls = formats.all_xpaths_results(
             remote_item_elem,
-            remote_format.DOWNLOAD_ITEM_CONTENT_URLS_XPATHS,
+            remote_format.DOWNLOAD_ITEM_ENCLOSURE_URLS_XPATHS,
         )
         try:
             # Download enclosures and assets only for this item.
-            item_download_asset_paths = self.download_urls(
-                item_download_asset_urls,
+            item_asset_paths = self.download_urls(
+                item_asset_urls,
             )
-            item_download_content_paths = self.download_urls(
-                item_download_content_urls,
+            item_enclosure_paths = self.download_urls(
+                item_enclosure_urls,
             )
-        except Exception:  # pragma: no cover, pylint: disable=broad-except
+        except Exception:  # pylint: disable=broad-except
             logger.exception(
                 "Problem downloading item URLs, continuing to next: %r",
                 remote_item_id,
             )
-            if utils.POST_MORTEM:
-                raise
+            if utils.POST_MORTEM:  # pragma: no cover
+                pdb.post_mortem()
             return None
-        return item_download_asset_paths, item_download_content_paths
+        return item_asset_paths, item_enclosure_paths
 
     def update_self(self, feed_format, archive_root):
         """
@@ -250,20 +333,40 @@ class ArchiveFeed:
         Also do any pre-processing needed to start updating the archive.
         """
         logger.debug("Parsing remote XML: %r", self.url)
-        remote_root = etree.fromstring(
+        remote_root = etree.fromstring(  # nosec: B320
             remote_response.content,
-            base_url=remote_response.url,
+            base_url=str(remote_response.url),
+            parser=utils.XML_PARSER,
         )
         return etree.ElementTree(remote_root)
 
-    def load_archive_tree(self, remote_format, remote_tree, download_paths):
+    def find_archive_path(self):
+        """
+        Locate this feed's file in the archive.
+        """
+        path = self.archive.root_path / self.archive.url_to_path(self.url)
+        if not path.exists():
+            archive_files = []
+            for archive_file in path.parent.glob(f"{path.stem}.*"):
+                guessed_type, _ = mimetypes.guess_type(archive_file)
+                if guessed_type is not None and (
+                    guessed_type.endswith("/xml") or guessed_type.endswith("+xml")
+                ):
+                    archive_files.append(archive_file)
+            if not archive_files:
+                raise ValueError(f"Could not locate feed in archive: {self.url}")
+            if len(archive_files) > 1:
+                logger.warning(
+                    "Multiple XML files found for feed, using first: %s\n%s",
+                    self.url,
+                    "\n  ".join([str(archive_file) for archive_file in archive_files]),
+                )
+            path = archive_files[0]
+        return path
+
+    def load_archive_tree(self):
         """
         Parse the local feed XML in the archive and return the tree.
-
-        If there is no local feed XML in the archive, such as the first time the feed is
-        updated, then initialize the archive tree from the remote tree.
-
-        Also do any pre-processing needed to start updating the archive.
         """
         archive_tree = None
         if (
@@ -277,16 +380,32 @@ class ArchiveFeed:
                 # there are errors parsing it, then treat it as if it's the first time
                 # archiving this feed.
                 try:
-                    archive_tree = etree.parse(feed_archive_opened)
+                    archive_tree = etree.parse(  # nosec B320
+                        feed_archive_opened,
+                        parser=utils.XML_PARSER,
+                    )
                 except SyntaxError:
                     logger.exception(
                         "Unhandled exception parsing archive feed: %r",
                         self.url,
                     )
                     if utils.POST_MORTEM:  # pragma: no cover
-                        raise
-            if archive_tree is not None:
-                archive_format = formats.FeedFormat.from_tree(self, remote_tree)
+                        pdb.post_mortem()
+        return archive_tree
+
+    def get_archive_tree(self, remote_tree, download_paths, remote_format=None):
+        """
+        Parse the archive feed XML if it exists or initialize an empty archive feed.
+
+        If there is no local feed XML in the archive, such as the first time the feed is
+        updated, then initialize the archive tree from the remote tree.
+
+        Also do any pre-processing needed to start updating the archive.
+        """
+        archive_tree = self.load_archive_tree()
+        if archive_tree is not None:
+            if remote_format is not None:  # pragma: no cover
+                archive_format = formats.FeedFormat.from_tree(self, archive_tree)
                 if not isinstance(
                     archive_format,
                     type(remote_format),
@@ -296,9 +415,9 @@ class ArchiveFeed:
                         "different from archive format, "
                         f"{type(archive_format).__name__!r}."
                     )
-                archive_root = archive_tree.getroot()
-                archived_items_parent = remote_format.get_items_parent(archive_root)
-        if archive_tree is None:
+            archive_root = archive_tree.getroot()
+            archived_items_parent = remote_format.get_items_parent(archive_root)
+        else:
             # First time requesting this feed, copy the remote feed, minus the items to
             # the archive and download the feed-level assets.
             logger.info(
@@ -345,7 +464,11 @@ class ArchiveFeed:
 
         return archive_tree
 
-    def download_urls(self, url_results):  # pylint: disable=too-many-branches
+    # TODO: Refactor to reduce complexity and improve readability and testibility
+    def download_urls(  # noqa: MC0001
+        self,
+        url_results,
+    ):  # pylint: disable=too-many-branches
         """
         Escape URLs to archive paths, download if new, and update URLs.
         """
@@ -367,18 +490,16 @@ class ArchiveFeed:
                 # Download the URL to the escaped local path in the archive
                 try:
                     download_path = self.download_url(url_result)
-                except (
-                    Exception  # pylint: disable=broad-except
-                ) as exc:  # pragma: no cover
+                except Exception as exc:  # pylint: disable=broad-except
                     excs[url_result] = exc
                     logger.exception(
                         "Problem downloading URL, removing from archive: %r",
                         url_result,
                     )
-                    if download_path is not None:
+                    if download_path is not None:  # pragma: no cover
                         download_path.unlink()
                     if utils.POST_MORTEM:  # pragma: no cover
-                        raise
+                        pdb.post_mortem()
                     continue
                 downloaded_paths[url_result] = download_path.relative_to(
                     self.archive.root_path,
@@ -387,49 +508,43 @@ class ArchiveFeed:
             # Update the URL in the feed XML to the relative archive path.
             # Update only after successful download to minimize inconsistent state on
             # errors.
-            if hasattr(url_result, "getparent") and hasattr(url_result, "attrname"):
-                url_parent = url_result.getparent()
-                download_relative = downloaded_paths[url_result]
-                if download_relative.name == self.archive.INDEX_BASENAME:
-                    download_relative = download_relative.parent
-                download_url_split = self.archive.url_split._replace(
-                    # Let pathlib normalize the relative path
-                    path=str(
-                        pathlib.PurePosixPath(self.archive.url_split.path)
-                        / urllib.parse.quote(str(download_relative))
-                    ),
+            url_parent = url_result.getparent()
+            download_relative = downloaded_paths[url_result]
+            if download_relative.name == self.archive.INDEX_BASENAME:
+                download_relative = download_relative.parent
+            download_url_split = self.archive.url_split._replace(
+                # Let pathlib normalize the relative path
+                path=str(
+                    pathlib.PurePosixPath(self.archive.url_split.path)
+                    / urllib.parse.quote(str(download_relative))
+                ),
+            )
+            if url_result.attrname:
+                logger.debug(
+                    'Updating feed URL: <%s %s="%s"...>',
+                    url_result.getparent().tag,
+                    url_result.attrname,
+                    download_url_split.geturl(),
                 )
-                if url_result.attrname:
-                    logger.debug(
-                        'Updating feed URL: <%s %s="%s"...>',
-                        url_result.getparent().tag,
-                        url_result.attrname,
-                        download_url_split.geturl(),
-                    )
-                    # Store the original remote URL in a namespace attribute
-                    url_parent.attrib[
-                        f"{{{self.NAMESPACE}}}attribute-{url_result.attrname}"
-                    ] = url_result
-                    # Update the archived URL to the local, relative URL
-                    url_parent.attrib[url_result.attrname] = download_url_split.geturl()
-                else:
-                    logger.debug(
-                        "Updating feed URL: <%s>%s</%s>",
-                        url_result.getparent().tag,
-                        download_url_split.geturl(),
-                        url_result.getparent().tag,
-                    )
-                    # Store the original remote URL in a namespace attribute
-                    url_parent.attrib[f"{{{self.NAMESPACE}}}text"] = url_result
-                    # Update the URL in the archive XML to the local, relative URL
-                    url_parent.text = download_url_split.geturl()
-            else:  # pragma: no coverx
-                raise NotImplementedError(
-                    f"Escaping URLs in {type(url_result)!r} text nodes"
-                    " not implemented yet",
+                # Store the original remote URL in a namespace attribute
+                url_parent.attrib[
+                    f"{{{self.NAMESPACE}}}attribute-{url_result.attrname}"
+                ] = url_result
+                # Update the archived URL to the local, relative URL
+                url_parent.attrib[url_result.attrname] = download_url_split.geturl()
+            else:
+                logger.debug(
+                    "Updating feed URL: <%s>%s</%s>",
+                    url_result.getparent().tag,
+                    download_url_split.geturl(),
+                    url_result.getparent().tag,
                 )
+                # Store the original remote URL in a namespace attribute
+                url_parent.attrib[f"{{{self.NAMESPACE}}}text"] = url_result
+                # Update the URL in the archive XML to the local, relative URL
+                url_parent.text = download_url_split.geturl()
 
-        if excs:  # pragma: no cover
+        if excs:
             # Ensure the feed item is processed again if any problem occurred with with
             # it's downloads
             raise list(excs.values())[0]
@@ -481,9 +596,9 @@ class ArchiveFeed:
         """
         logger.info("Downloading URL into archive: %r", url_result)
         url_split = self.resolve_url(url_result)
-        with self.archive.requests.get(
+        with self.archive.client.stream(
+            "GET",
             url_split.geturl(),
-            stream=True,
         ) as download_response:
             download_path = self.archive.root_path / self.archive.response_to_path(
                 download_response,
@@ -498,166 +613,185 @@ class ArchiveFeed:
                 return download_path
             logger.debug("Writing download into archive: %r", str(download_relative))
             download_path.parent.mkdir(parents=True, exist_ok=True)
-            stream.stream_response_to_file(download_response, path=download_path)
-        download_stat = download_path.stat()
+            with download_path.open("wb") as download_opened:
+                for data in download_response.iter_bytes():
+                    download_opened.write(data)
 
         update_download_metadata(download_response, download_path)
 
-        if (
-            download_response.headers.get(
-                "Content-Encoding",
-                "binary",
-            )
-            .strip()
-            .lower()
-            == "binary"
-            and "Content-Length" in download_response.headers
-        ):
-            try:
-                remote_content_length = int(
-                    download_response.headers["Content-Length"].strip(),
-                )
-            except ValueError:  # pragma: no cover
-                pass
-            else:
-                if download_stat.st_size != remote_content_length:  # pragma: no cover
-                    logger.error(
-                        "Downloaded content size different from remote: %r -> %r",
-                        download_stat.st_size,
-                        remote_content_length,
-                    )
-
         return download_path
 
-    def link_item_content(
+    def link_item_enclosures(
         self,
+        remote_format,
         feed_elem,
         item_elem,
-        item_content_paths,
-    ):  # pylint: disable=too-many-branches
+        item_enclosure_paths,
+    ):
         """
-        Link item content/enclosures into media library hierarchies using plugins.
+        Link item enclosures into media library hierarchies using plugins.
         """
-        content_link_paths = {}
+        link_paths = {}
+        if not self.enclosure_plugins and not self.enclosure_fallack_plugins:
+            # Avoid unnecessary work when no link plugins are configured, particularly
+            # parsing the item with `feedparser`.
+            return link_paths
+
+        feed_parsed = utils.parse_item_feed(remote_format, feed_elem, item_elem)
+        (item_parsed,) = feed_parsed.entries
         for (
             url_result,
-            content_archive_relative,
-        ) in item_content_paths.items():
-            basename = content_archive_relative.name
+            enclosure_path,
+        ) in item_enclosure_paths.items():
             link_idx = 0
-            for link_path_plugin in self.link_path_plugins:
-                for content_link_path in self.list_item_content_link_plugin_paths(
+            for enclosure_plugin in self.enclosure_plugins:
+                for link_path in self.list_plugin_enclosure_links(
                     feed_elem,
+                    feed_parsed,
                     item_elem,
+                    item_parsed,
                     url_result,
-                    basename,
-                    content_archive_relative,
-                    link_path_plugin,
+                    enclosure_path,
+                    enclosure_plugin,
                 ):
-                    content_link_paths.setdefault(url_result, []).append(
+                    link_paths.setdefault(url_result, []).append(
                         self.link_plugin_file(
                             url_result,
-                            content_archive_relative,
-                            content_link_path,
+                            enclosure_path,
+                            link_path,
                             link_idx,
                         )
                     )
                     link_idx += 1
-        return content_link_paths
+            if link_idx > 0:
+                # At least one plugin linked the enclosure, skip the fallbacks
+                continue
+            # Link and fallback configurations for this enclosure
+            for enclosure_plugin in self.enclosure_fallack_plugins:
+                for link_path in self.list_plugin_enclosure_links(
+                    feed_elem,
+                    feed_parsed,
+                    item_elem,
+                    item_parsed,
+                    url_result,
+                    enclosure_path,
+                    enclosure_plugin,
+                ):
+                    link_paths.setdefault(url_result, []).append(
+                        self.link_plugin_file(
+                            url_result,
+                            enclosure_path,
+                            link_path,
+                            link_idx,
+                        )
+                    )
+                    link_idx += 1
+        return link_paths
 
-    def list_item_content_link_plugin_paths(
+    def list_plugin_enclosure_links(
         self,
         feed_elem,
+        feed_parsed,
         item_elem,
+        item_parsed,
         url_result,
-        basename,
-        content_archive_relative,
-        link_path_plugin,
+        enclosure_path,
+        enclosure_plugin,
     ):  # pylint: disable=too-many-arguments
         """
-        Return the content link paths for an individual download and plugin.
+        Return the links for an individual enclosure and plugin.
         """
+        kwargs = {}
         match = None
-        if "match-re" in link_path_plugin.config:
+        if "match-re" in enclosure_plugin.config:
             match_kwargs = locals().copy()
             del match_kwargs["self"]
             match = self.link_item_plugin_match(**match_kwargs)
             if match is None:
                 return []
+            kwargs.update(match.groupdict())
 
         # Delegate to the plugin
         logger.debug(
-            "Linking item content with %r plugin: %s",
-            type(link_path_plugin),
-            str(content_archive_relative),
+            "Linking item enclosure with %r plugin: %s",
+            type(enclosure_plugin),
+            str(enclosure_path),
         )
         try:
-            content_link_strs = link_path_plugin(
+            link_strs = enclosure_plugin(
                 archive_feed=self,
                 feed_elem=feed_elem,
+                feed_parsed=feed_parsed,
                 item_elem=item_elem,
+                item_parsed=item_parsed,
                 url_result=url_result,
-                basename=basename,
+                enclosure_path=enclosure_path,
                 match=match,
+                **kwargs,
             )
-        except Exception:  # pragma: no cover, pylint: disable=broad-except
+        except Exception:  # pylint: disable=broad-except
             logger.exception(
-                "Problem linking item content with %r, continuing to next: %s",
-                type(link_path_plugin),
-                str(content_archive_relative),
+                "Problem linking item enclosure with %r, continuing to next: %s",
+                type(enclosure_plugin),
+                str(enclosure_path),
             )
             if utils.POST_MORTEM:  # pragma: no cover
-                raise
-            content_link_strs = []
-        if content_link_strs is None:  # pragma: no cover
+                pdb.post_mortem()
+            link_strs = []
+        if link_strs is None:  # pragma: no cover
             # Plugin handled any linking itself
-            content_link_strs = []
-        if isinstance(content_link_strs, str):  # pragma: no cover
-            content_link_strs = [content_link_strs]
+            link_strs = []
+        if isinstance(link_strs, str):  # pragma: no cover
+            link_strs = [link_strs]
+        # Filter out duplicate paths but preserve order
+        uniq_link_strs = []
+        for enclosure_link_str in link_strs:
+            if enclosure_link_str not in uniq_link_strs:
+                uniq_link_strs.append(enclosure_link_str)
         return [
             self.archive.root_path
             / pathlib.Path(
-                utils.quote_path(content_link_str),
+                utils.quote_path(enclosure_link_str),
             )
-            for content_link_str in content_link_strs
+            for enclosure_link_str in uniq_link_strs
         ]
 
     def link_item_plugin_match(self, **kwargs):
         """
         If configured, check for a regular expression match for a feed item enclosure.
         """
-        link_path_plugin = kwargs["link_path_plugin"]
+        enclosure_plugin = kwargs["enclosure_plugin"]
         kwargs["self"] = self
         try:
-            match_string = eval(  # pylint: disable=eval-used
-                f"f{link_path_plugin.config['match-string']!r}",
+            match_string = eval(  # nosec B307, pylint: disable=eval-used
+                f"f{enclosure_plugin.config['match-string']!r}",
                 globals(),
                 kwargs,
             )
-        except Exception:  # pragma: no cover, pylint: disable=broad-except
+        except Exception:  # pylint: disable=broad-except
             logger.exception(
                 "Problem expanding `match-string` template for %r: %r",
-                type(link_path_plugin),
-                link_path_plugin.config["match-string"],
+                type(enclosure_plugin),
+                enclosure_plugin.config["match-string"],
             )
             if utils.POST_MORTEM:  # pragma: no cover
-                raise
+                pdb.post_mortem()
             return None
         try:
-            match = link_path_plugin.config["match-re"].match(match_string)
+            match = enclosure_plugin.config["match-re"].match(match_string)
         except Exception:  # pragma: no cover, pylint: disable=broad-except
             logger.exception(
                 "Problem matching `match-pattern` for %r: %r",
-                type(link_path_plugin),
-                link_path_plugin.config["match-string"],
+                type(enclosure_plugin),
+                enclosure_plugin.config["match-string"],
             )
             if utils.POST_MORTEM:  # pragma: no cover
-                raise
+                pdb.post_mortem()
             return None
-        if match is None:  # pragma: no cover
-            logger.info(
+        if match is None:
+            logger.debug(
                 "The %r plugin `match-pattern` did not match: %r",
-                type(link_path_plugin),
+                type(enclosure_plugin),
                 match_string,
             )
             return None
@@ -666,57 +800,57 @@ class ArchiveFeed:
     def link_plugin_file(
         self,
         url_result,
-        content_archive_relative,
-        content_link_path,
+        enclosure_archive_relative,
+        link_path,
         link_idx,
     ):
         """
-        Link an item content/enclosure to a filesystem path returned by a plugin.
+        Link an item enclosure to a filesystem path returned by a plugin.
         """
         # Make the link relative
-        content_link_target = pathlib.Path(
+        enclosure_link_target = pathlib.Path(
             os.path.relpath(
-                self.archive.root_path / content_archive_relative,
-                content_link_path.parent,
+                self.archive.root_path / enclosure_archive_relative,
+                link_path.parent,
             ),
         )
 
-        # Append numerical index if there are multiple content downloads for this
+        # Append numerical index if there are multiple enclosure downloads for this
         # item.
-        content_link_stem = content_link_path.stem
-        content_index = 0
-        while os.path.lexists(content_link_path):
-            if (
-                content_link_path.is_symlink()
-                and content_link_path.readlink() == content_link_target
+        enclosure_link_stem = link_path.stem
+        enclosure_index = 0
+        while os.path.lexists(link_path):
+            if link_path.is_symlink() and os.readlink(link_path) == str(
+                enclosure_link_target
             ):
                 logger.debug(
-                    "Duplicate item URL, skip content link: %r -> %r",
-                    str(content_link_path),
-                    str(content_link_target),
+                    "Duplicate item URL, skip enclosure link: %r -> %r",
+                    str(link_path),
+                    str(enclosure_link_target),
                 )
                 break
-            content_index += 1
-            content_link_path = content_link_path.with_stem(
-                content_link_stem[
+            enclosure_index += 1
+            link_path = link_path.with_name(
+                enclosure_link_stem[
                     : self.archive.root_stat.f_namemax
-                    - (len(content_link_path.suffix) + len(str(content_index)))
+                    - (len(link_path.suffix) + len(str(enclosure_index)))
                 ]
-                + str(content_index),
+                + str(enclosure_index)
+                + link_path.suffix,
             )
         else:
             logger.info(
-                "Linking item content: %r -> %r",
-                str(content_link_path),
-                str(content_link_target),
+                "Linking item enclosure: %r -> %r",
+                str(link_path),
+                str(enclosure_link_target),
             )
-            content_link_path.parent.mkdir(parents=True, exist_ok=True)
-            content_link_path.symlink_to(content_link_target)
+            link_path.parent.mkdir(parents=True, exist_ok=True)
+            link_path.symlink_to(enclosure_link_target)
 
         url_result.getparent().attrib[
-            f"{{{self.NAMESPACE}}}content-link-{link_idx}"
-        ] = str(content_link_path)
-        return content_link_path
+            f"{{{self.NAMESPACE}}}enclosure-link-{link_idx}"
+        ] = str(link_path)
+        return link_path
 
 
 def update_download_metadata(download_response, download_path):
