@@ -7,9 +7,10 @@ import datetime
 import pathlib
 import urllib
 
-from lxml import etree
-import requests_mock
+from lxml import etree  # nosec: B410
+import respx.models
 
+from .. import utils
 from .. import tests
 
 
@@ -24,8 +25,8 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
         """
         # Put the mocks in place
         self.update_feed(self.archive_feed)
-        with self.assertRaises(requests_mock.exceptions.NoMockAddress):
-            self.archive.requests.get("http://example.com")
+        with self.assertRaises(respx.models.AllMockedAssertionError):
+            self.archive.client.get("http://example.com")
 
     def test_download_file_metadata(self):
         """
@@ -49,10 +50,10 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
 
         # Download the enclosure into the archive
         orig_request_mocks = self.mock_remote(self.archive_feed)
-        redirect_request_mock = self.requests_mock.get(
+        redirect_request_mock = self.client_mock.get(
             self.ENCLOSURE_URL,
+        ).respond(
             status_code=302,
-            reason="Found",
             headers={"Location": self.ENCLOSURE_REDIRECT_URL},
         )
         self.archive_feed.update()
@@ -64,12 +65,6 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
         )
 
         # The most appropriate file basename is symlinked to the download file
-        _, download_request_mock = orig_request_mocks[self.ENCLOSURE_URL]
-        self.assertEqual(
-            download_request_mock.call_count,
-            0,
-            "Mock request without redirect response called",
-        )
         self.assertEqual(
             redirect_request_mock.call_count,
             1,
@@ -85,10 +80,12 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
         # Test in the absence of the response headers
         self.archive_feed.path.unlink()
         enclosure_archive_path.unlink()
-        no_header_request_mock = self.requests_mock.get(
+        no_header_request_mock = self.client_mock.get(
             self.ENCLOSURE_URL,
+        ).respond(
             content=self.ENCLOSURE_MOCK_PATH.read_bytes(),
         )
+        no_header_request_mock.reset()
         self.assert_no_header_download_mtime(
             no_header_request_mock,
             self.archive.root_path / self.ENCLOSURE_RELATIVE.with_suffix(".mp3"),
@@ -171,7 +168,10 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
         )
 
         # Assert archive URLs updated
-        archive_tree = etree.parse(self.archive_feed.path.open())
+        archive_tree = etree.parse(  # nosec: B320
+            self.archive_feed.path.open(),
+            parser=utils.XML_PARSER,
+        )
         feed_link_split = urllib.parse.urlsplit(
             archive_tree.find("channel").find("link").text,
         )
@@ -203,7 +203,8 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
             "Wrong feed image absolute URL in feed",
         )
         archive_items = archive_tree.find("channel").findall("item")
-        item_link_split = urllib.parse.urlsplit(archive_items[0].find("link").text)
+        archive_item = archive_items[1]
+        item_link_split = urllib.parse.urlsplit(archive_item.find("link").text)
         item_link_path = (
             pathlib.PurePosixPath(
                 urllib.parse.unquote(item_link_split.path).lstrip("/")
@@ -215,7 +216,7 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
             "Wrong item link index HTML absolute URL in feed",
         )
         item_enclosure_split = urllib.parse.urlsplit(
-            archive_items[0].find("enclosure").attrib["url"],
+            archive_item.find("enclosure").attrib["url"],
         )
         item_enclosure_path = pathlib.PurePosixPath(
             urllib.parse.unquote(item_enclosure_split.path).lstrip("/"),
@@ -225,7 +226,7 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
             "Wrong item enclosure absolute URL in feed",
         )
         item_media_content_split = urllib.parse.urlsplit(
-            archive_items[0].find("{*}content").attrib["url"],
+            archive_item.find("{*}content").attrib["url"],
         )
         item_media_content_path = pathlib.PurePosixPath(
             urllib.parse.unquote(item_media_content_split.path).lstrip("/"),
@@ -236,7 +237,7 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
             "Wrong item media content absolute URL in feed",
         )
         item_image_split = urllib.parse.urlsplit(
-            archive_items[0].find("{*}image").attrib["href"],
+            archive_item.find("{*}image").attrib["href"],
         )
         item_image_path = pathlib.PurePosixPath(
             urllib.parse.unquote(item_image_split.path).lstrip("/"),
@@ -255,7 +256,6 @@ class FeedarchiverDownloadTests(tests.FeedarchiverDownloadsTestCase):
                 msg="Test one feed download",
                 archive_relative=str(archive_relative),
             ):
-
                 # Assert that the request mock was called correctly
                 download_url, mock_path = self.archive_relative_to_remote_url(
                     archive_relative,
