@@ -6,8 +6,9 @@ import os
 import datetime
 import logging
 
-from lxml import etree
+from lxml import etree  # nosec: B410
 
+from .. import utils
 from .. import formats
 from .. import feed
 from .. import tests
@@ -24,16 +25,16 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
         / tests.FeedarchiverTestCase.FEED_ARCHIVE_RELATIVE
     )
     FEED_FORMAT_PARAMS = [
-        dict(
-            feed_format_class=formats.RssFeedFormat,
-            relative_path=FEED_MOCK_RELATIVE,
-            items_parent_tag="channel",
-            item_tag="item",
-            item_id="7bd204c6-1655-4c27-aeee-53f933c5395f",
-        ),
-        dict(
-            feed_format_class=formats.AtomFeedFormat,
-            relative_path=(
+        {
+            "feed_format_class": formats.RssFeedFormat,
+            "relative_path": FEED_MOCK_RELATIVE,
+            "items_parent_tag": "channel",
+            "item_tag": "item",
+            "item_id": "7bd204c6-1655-4c27-aeee-53f933c5395f",
+        },
+        {
+            "feed_format_class": formats.AtomFeedFormat,
+            "relative_path": (
                 tests.FeedarchiverTestCase.EXAMPLE_RELATIVE
                 / tests.FeedarchiverTestCase.REMOTE_MOCK
                 / "https"
@@ -41,12 +42,12 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
                 / "feeds"
                 / "waldo"
             ),
-            items_parent_tag=(
+            "items_parent_tag": (
                 f"{{http://www.w3.org/2005/Atom}}{formats.AtomFeedFormat.ROOT_TAG}"
             ),
-            item_tag="entry",
-            item_id="urn:uuid:1225c695-cfb8-4ebb-aaaa-80da344efa6a",
-        ),
+            "item_tag": "entry",
+            "item_id": "urn:uuid:1225c695-cfb8-4ebb-aaaa-80da344efa6a",
+        },
     ]
 
     def test_feed_configs_requested(self):
@@ -68,14 +69,20 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
             self.feed_path.is_file(),
             "Archive of feed XML does not exist after updating",
         )
-        with feed_path.open() as remote_opened:
-            remote_tree = etree.parse(remote_opened)
+        with feed_path.open(encoding="utf-8") as remote_opened:
+            remote_tree = etree.parse(  # nosec: B320
+                remote_opened,
+                parser=utils.XML_PARSER,
+            )
         remote_items = remote_tree.find("channel").iterchildren("item")
         remote_item_ids = [
             remote_item.find("guid").text for remote_item in remote_items
         ]
-        with self.feed_path.open() as archive_opened:
-            archive_tree = etree.parse(archive_opened)
+        with self.feed_path.open(encoding="utf-8") as archive_opened:
+            archive_tree = etree.parse(  # nosec: B320
+                archive_opened,
+                parser=utils.XML_PARSER,
+            )
         archive_items = archive_tree.find("channel").iterchildren("item")
         archive_item_ids = [
             archive_item.find("guid").text for archive_item in archive_items
@@ -107,14 +114,20 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
             2,
             "Wrong number of original feed URL requests",
         )
-        with feed_path.open() as remote_opened:
-            remote_tree = etree.parse(remote_opened)
+        with feed_path.open(encoding="utf-8") as remote_opened:
+            remote_tree = etree.parse(  # nosec: B320
+                remote_opened,
+                parser=utils.XML_PARSER,
+            )
         remote_items = remote_tree.find("channel").iterchildren("item")
         remote_item_ids = [
             remote_item.find("guid").text for remote_item in remote_items
         ]
-        with self.feed_path.open() as archive_opened:
-            archive_tree = etree.parse(archive_opened)
+        with self.feed_path.open(encoding="utf-8") as archive_opened:
+            archive_tree = etree.parse(  # nosec: B320
+                archive_opened,
+                parser=utils.XML_PARSER,
+            )
         archive_items = archive_tree.find("channel").iterchildren("item")
         archive_item_ids = [
             archive_item.find("guid").text for archive_item in archive_items
@@ -140,6 +153,7 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
         )
 
         # Update the archive after the remote feed is updated with a new item added
+        get_mock.reset()
         added_item_request_mocks, _ = self.update_feed(
             archive_feed=self.archive_feed,
             remote_mock="added-item",
@@ -178,6 +192,7 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
         _, added_item_get_mock = added_item_request_mocks[self.feed_url]
 
         # Update the archive after the remote feed is updated with an item removed
+        added_item_get_mock.reset()
         removed_item_request_mocks, _ = self.update_feed(
             archive_feed=self.archive_feed,
             remote_mock="removed-item",
@@ -220,9 +235,16 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
         )
         reordered_item_feed_path, _ = reordered_item_request_mocks[self.feed_url]
 
-        remote_tree = etree.parse(reordered_item_feed_path.open())
+        remote_tree = etree.parse(  # nosec: B320
+            reordered_item_feed_path.open(encoding="utf-8"),
+            parser=utils.XML_PARSER,
+        )
         remote_items = remote_tree.find("channel").findall("item")
-        archive_tree = etree.parse(self.feed_path.open())
+        with self.feed_path.open(encoding="utf-8") as feed_opened:
+            archive_tree = etree.parse(  # nosec: B320
+                feed_opened,
+                parser=utils.XML_PARSER,
+            )
         archive_items = archive_tree.find("channel").findall("item")
         self.assertEqual(
             archive_items[0].find("guid").text,
@@ -242,14 +264,18 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
 
     def test_feed_empty(self):
         """
-        Updating an empty feed works without any errors
+        Updating an empty feed works without any errors.
         """
         # Populate with a feed containing multiple items
         self.update_feed(
             archive_feed=self.archive_feed,
             remote_mock="empty",
         )
-        archive_tree = etree.parse(self.feed_path.open())
+        with self.feed_path.open(encoding="utf-8") as feed_opened:
+            archive_tree = etree.parse(  # nosec: B320
+                feed_opened,
+                parser=utils.XML_PARSER,
+            )
         archive_items = archive_tree.find("channel").findall("item")
         self.assertEqual(
             archive_items,
@@ -268,8 +294,13 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
                 msg="Test one XML feed format",
                 feed_format_class=feed_format_class,
             ):
-                with (self.REMOTES_PATH / relative_path).open() as feed_opened:
-                    feed_tree = etree.parse(feed_opened)
+                with (self.REMOTES_PATH / relative_path).open(
+                    encoding="utf-8"
+                ) as feed_opened:
+                    feed_tree = etree.parse(  # nosec: B320
+                        feed_opened,
+                        parser=utils.XML_PARSER,
+                    )
                 feed_root = feed_tree.getroot()
                 feed_format = feed_format_class(self.archive_feed)
 
@@ -332,10 +363,12 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
             / "added-item"
             / tests.FeedarchiverTestCase.FEED_ARCHIVE_RELATIVE
         )
-        no_header_request_mock = self.requests_mock.get(
+        no_header_request_mock = self.client_mock.get(
             self.feed_url,
+        ).respond(
             content=no_header_feed_mock_path.read_bytes(),
         )
+        no_header_request_mock.reset()
         self.assert_no_header_download_mtime(
             no_header_request_mock,
             self.feed_path,
@@ -356,7 +389,10 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
         )
 
         # The archive version of the feed was somehow written as an empty file.
-        self.feed_path.write_text("{'status': 'Got JSON instead of RSS'")
+        self.feed_path.write_text(
+            "{'status': 'Got JSON instead of RSS'",
+            encoding="utf-8",
+        )
         with self.assertLogs(
             feed.logger,
             level=logging.ERROR,
@@ -370,7 +406,7 @@ class FeedarchiverFeedTests(tests.FeedarchiverTestCase):
         self.assertEqual(
             len(logged_msgs.records),
             1,
-            "Wrong number of download item logged records",
+            "Wrong number of logged feed records",
         )
         self.assertIn(
             "Unhandled exception parsing archive feed",
